from pathlib import Path
from typing import List, Optional

from pydantic import BaseModel, Field


class Project(BaseModel):
    name: str
    profile: Optional[str]
    models_paths: Optional[List[str]] = Field(alias="models-paths")
    seed_paths: Optional[List[str]] = Field(alias="seed-paths")
    test_paths: Optional[List[str]] = Field(alias="test-paths")
    analysis_paths: Optional[List[str]] = Field(alias="analysis-paths")
    macro_paths: Optional[List[str]] = Field(alias="macro-paths")
    snapshot_paths: Optional[List[str]] = Field(alias="snapshot-paths")
    docs_paths: Optional[List[str]] = Field(alias="docs-paths")
    asset_paths: Optional[List[str]] = Field(alias="asset-paths")
    target_path: Optional[Path] = Field(alias="target-path")



from pathlib import Path

from odd_dbt.utils import load_json


class Manifest:
    def __init__(self, file: Path) -> None:
        self._manifest = load_json(file)

    @property
    def nodes(self):
        return self._manifest["nodes"]



from typing import Optional

from funcy import first


class Result:
    def __init__(self, result: dict) -> None:
        self._result = result

    @property
    def unique_id(self) -> str:
        return self._result["unique_id"]

    @property
    def execution_period(self) -> tuple[Optional[str], Optional[str]]:
        execution = first(
            period for period in self._result["timing"] if period["name"] == "execute"
        )

        if execution is None:
            return None, None

        return execution.get("started_at"), execution.get("completed_at")

    @property
    def status(self) -> str:
        return self._result["status"]

    @property
    def status_reason(self) -> Optional[str]:
        return self._result.get("message")



import subprocess
import traceback
from pathlib import Path
from typing import Optional

import typer
from odd_models.api_client.v2.odd_api_client import Client
from oddrn_generator import DbtGenerator

from odd_dbt.context import DbtContext
from odd_dbt.logger import logger
from odd_dbt.mapper.dbt_test import DbtTestMapper

app = typer.Typer(
    short_help="Run dbt tests and inject results to ODD platform",
    pretty_exceptions_show_locals=False,
)


@app.command()
def test(
    project_dir: Path = typer.Option(default=Path.cwd()),
    profiles_dir: Path = typer.Option(default=None),
    target: Optional[str] = typer.Option(default=None),
    profile: Optional[str] = typer.Option(default=None),
    platform_host: str = typer.Option(..., "--host", "-h", envvar="ODD_PLATFORM_HOST"),
    platform_token: str = typer.Option(
        ..., "--token", "-t", envvar="ODD_PLATFORM_TOKEN"
    ),
    dbt_data_source_oddrn: str = typer.Option(
        default=None,
    ),
    dbt_host: str = typer.Option(default="localhost"),
):
    logger.info("Start dbt test process. Version: 0.1.21")
    try:
        logger.info(
            f"Run dbt process with {project_dir=}, {profiles_dir=}, {target=}, {profile=}"
        )

        args = [
            "dbt",
            "test",
        ]

        if project_dir:
            args.extend(["--project-dir", project_dir])

        if profiles_dir:
            args.extend(["--profiles-dir", profiles_dir])

        if profile:
            args.extend(["--profile", profile])

        if target:
            args.extend(["--target", target])

        process = subprocess.run(args)

        if process.returncode >= 2:
            logger.error("Dbt test command failed")
            raise typer.Exit(2)

        client = Client(host=platform_host, token=platform_token)

        generator = DbtGenerator(host_settings=dbt_host)

        dbt_data_source_oddrn = dbt_data_source_oddrn
        if not dbt_data_source_oddrn:
            dbt_data_source_oddrn = generator.get_data_source_oddrn()
            logger.info(f"Creating data source for dbt: ODDRN={dbt_data_source_oddrn}")
            client.create_data_source(
                data_source_name="dbt",
                data_source_oddrn=dbt_data_source_oddrn,
            )

        context = DbtContext(project_dir=project_dir, profile=profile, target=target)

        logger.info("Got DBT test context. Start mapping...")
        data_entities = DbtTestMapper(context=context, generator=generator).map()

        logger.info("Mapping finished. Start injecting...")
        client.ingest_data_entity_list(data_entities=data_entities)

        logger.info("Injecting finished.")
    except Exception as e:
        logger.debug(traceback.format_exc())
        logger.error(e)
        raise typer.Exit(1)


if __name__ == "__main__":
    app()



import re
from typing import Optional

from dbt.contracts.graph.nodes import (
    GenericTestNode,
    SingularTestNode,
    TestMetadata,
    TestNode,
)
from funcy import partial

from ..errors import CantParseReason
from ..logger import logger


def from_kwargs(test_metadata: TestMetadata, field: str, default=None):
    return test_metadata.kwargs.get(field, default)


get_column_name = partial(from_kwargs, field="column_name", default="Unknown column")
get_to = partial(from_kwargs, field="to")
get_field = partial(from_kwargs, field="field")
get_expression = partial(from_kwargs, field="expression")
get_compare_model = partial(from_kwargs, field="compare_model")
get_from_condition = partial(from_kwargs, field="from_condition")
get_to_condition = partial(from_kwargs, field="to_condition")
get_compare_column = partial(from_kwargs, field="compare_column", default=[])


def get_model_name(test_metadata: TestMetadata) -> Optional[str]:
    try:
        if model := from_kwargs(test_metadata, field="model"):
            return parse_model_name(model)
        else:
            return None
    except Exception as e:
        logger.warning(f"Can't get model name from {test_metadata=}. {e}")
        return None


def parse_model_name(model_name: Optional[str]) -> Optional[str]:
    if model_name is None:
        return None

    try:
        result = re.search(r"(\w+)", model_name)
        return model_name if result is None else result[1]
    except Exception as e:
        logger.warning(f"Can't parse model name {model_name=}. {e}")
        return model_name


class GenericTestReason:
    def get_reason(self, test_node: GenericTestNode) -> str:
        try:
            test_metadata = test_node.test_metadata

            return getattr(self, test_metadata.name)(test_node.test_metadata)
        except Exception as e:
            raise CantParseReason(f"Cant parse GenericTestReason. {e}") from e

    def unique(self, test_metadata: TestMetadata) -> str:
        column = get_column_name(test_metadata)
        model = get_model_name(test_metadata)

        return f"The {column=} in the {model=} must be unique"

    def not_null(self, test_metadata: TestMetadata) -> str:
        column = get_column_name(test_metadata)
        model = get_model_name(test_metadata)

        return f"The {column=} in the {model=} must not contain null values"

    def accepted_values(self, test_metadata: TestMetadata) -> str:
        column = get_column_name(test_metadata)
        model = get_model_name(test_metadata)
        acc_values = test_metadata.kwargs.get("values", "Unknown values")

        return f"The {column} in the {model=} must be one of {acc_values}"

    def relationships(self, test_metadata: TestMetadata) -> str:
        column = get_column_name(test_metadata)
        model = get_model_name(test_metadata)

        ref_model = parse_model_name(get_to(test_metadata))
        ref_field = get_field(test_metadata, "id")

        return f"Each value in the {column} in the {model} should exists as an {ref_field} in the {ref_model}"

    def expression_is_true(self, test_metadata: TestMetadata) -> str:
        expression = get_expression(test_metadata) or "Unknown expression"
        if len(expression) > 200:
            expression = f"{expression[:200]}..."
        return f"{expression=} is not true"

    def at_least_one(self, test_metadata: TestMetadata) -> str:
        return f"At least one row is expected for {get_column_name(test_metadata)}"

    def cardinality_equality(self, test_metadata: TestMetadata) -> str:
        return f"Cardinality of {get_column_name(test_metadata)} is not as expected"

    def equal_rowcount(self, test_metadata: TestMetadata) -> str:
        model = get_model_name(test_metadata)
        compare_model = parse_model_name(get_compare_model(test_metadata))
        return f"Rows count of {model=} and {compare_model=} are not equal"

    def equality(self, test_metadata: TestMetadata) -> str:
        model = get_model_name(test_metadata)
        compare_model = parse_model_name(get_compare_model(test_metadata))
        compare_column = get_compare_column(test_metadata) or []
        columns = ",".join(compare_column)
        if len(columns) > 200:
            columns = f"{columns[:200]}..."
        return f"{model} columns {columns} are not equal with {compare_model=}"

    def not_constant(self, test_metadata: TestMetadata) -> str:
        return f"{get_column_name(test_metadata)} is constant"

    def recency(self, test_metadata: TestMetadata) -> str:
        return f"{get_column_name(test_metadata)} is not recent enough"

    def relationships_where(self, test_metadata: TestMetadata) -> str:
        column = get_column_name(test_metadata)
        model = get_model_name(test_metadata)
        target_column = get_field(test_metadata, "id")
        target_model = parse_model_name(get_to(test_metadata))
        from_condition = get_from_condition(test_metadata)
        to_condition = get_to_condition(test_metadata)

        return f"{column=} in {model=} with {from_condition} is not related to {target_column=} in {target_model=} with {to_condition=}"

    def unique_combination_of_columns(self, test_metadata: TestMetadata) -> str:
        columns = test_metadata.kwargs.get("combination_of_columns", [])
        if len(columns) > 200:
            columns = f"{columns[:200]}..."
        return f"Combination of {columns} is not unique"


class SingularTestReason:
    def get_reason(self, test_node: SingularTestNode) -> str:
        raise CantParseReason("Reason for SingularTestNode not implemented yet.")


class StatusReason:
    def get_reason(self, test_node: TestNode) -> str:
        try:
            if isinstance(test_node, GenericTestNode):
                return GenericTestReason().get_reason(test_node)
            elif isinstance(test_node, SingularTestNode):
                return SingularTestReason().get_reason(test_node)
            else:
                return self.unknown_error()
        except Exception as e:
            logger.warning(f"Failed to get reason for test: {e}")
            logger.debug(f"Test definition: {test_node=}")
            return self.unknown_error()

    def unknown_error(self) -> str:
        return "Unknown reason"



from datetime import datetime
from typing import Iterable, Optional

import pytz
from dbt.contracts.graph.nodes import ParsedNode, TestNode
from funcy import lkeep, walk_values
from odd_models.models import (
    DataEntity,
    DataEntityList,
    DataEntityType,
    DataQualityTest,
    DataQualityTestExpectation,
    DataQualityTestRun,
    QualityRunStatus,
)
from oddrn_generator import DbtGenerator

from odd_dbt.context import DbtContext
from odd_dbt.mapper.data_source import DataSource, get_datasource_generator
from odd_dbt.mapper.metadata import get_metadata
from odd_dbt.mapper.status_reason import StatusReason
from odd_dbt.models import Result

from ..logger import logger


class DbtTestMapper:
    def __init__(self, context: DbtContext, generator: DbtGenerator) -> None:
        self._context = context
        self._generator = generator

    def map(self) -> DataEntityList:
        data_entities = []
        all_nodes = walk_values(ParsedNode._deserialize, self._context.manifest.nodes)

        for result in self._context.results:
            try:
                data_entities.extend(self.map_result(result, all_nodes))
            except Exception as e:
                logger.warning(f"Can't map result {result.unique_id}: {e}")
                continue

        data_entities = DataEntityList(
            data_source_oddrn=self._generator.get_data_source_oddrn(),
            items=data_entities,
        )

        logger.debug(data_entities.json(exclude_none=True))
        return data_entities

    def map_result(
        self, result: Result, all_nodes
    ) -> Optional[tuple[DataEntity, DataEntity]]:
        test_id: str = result.unique_id
        invocation_id: str = self._context.invocation_id

        assert test_id is not None
        assert invocation_id is not None

        start_time, end_time = result.execution_period
        test_node: TestNode = all_nodes[test_id]

        if test_node.resource_type != "test":
            logger.warning(f"Node {test_id} is not a test node")
            raise ValueError(f"Node {test_id} is not a test node")

        job = self.map_config(test_node, all_nodes)

        oddrn = self._generator.get_oddrn_by_path("runs", f"{invocation_id}")
        status, status_reason = parse_status(result, test_node)

        name = test_node.name
        if len(name) > 250:
            name = test_node.alias

        run = DataEntity(
            oddrn=oddrn,
            name=name,
            type=DataEntityType.JOB_RUN,
            owner=None,
            data_quality_test_run=DataQualityTestRun(
                data_quality_test_oddrn=job.oddrn,
                start_time=datetime_format(start_time) or datetime.now(tz=pytz.UTC),
                end_time=datetime_format(end_time) or datetime.now(tz=pytz.UTC),
                status=status,
                status_reason=status_reason,
            ),
        )

        return job, run

    def map_config(
        self, test_node: TestNode, nodes: dict[str, ParsedNode]
    ) -> DataEntity:
        dataset_list = lkeep([*self.get_dataset_oddrn(test_node, nodes)])
        assert len(dataset_list) > 0

        name = test_node.name
        if len(name) > 120:
            name = test_node.alias

        self._generator.set_oddrn_paths(
            **{"databases": test_node.database, "tests": name}
        )

        return DataEntity(
            oddrn=self._generator.get_oddrn_by_path("tests"),
            owner=None,
            name=name,
            type=DataEntityType.JOB,
            metadata=[get_metadata(test_node)],
            data_quality_test=DataQualityTest(
                suite_name=test_node.name,
                dataset_list=dataset_list,
                expectation=DataQualityTestExpectation(
                    type=test_node.test_metadata.name
                ),
            ),
        )

    def get_dataset_oddrn(
        self, test_node: TestNode, nodes: dict[str, ParsedNode]
    ) -> Iterable[Optional[str]]:
        data_source = DataSource(self._context.profile.type)

        if test_node.test_node_type == "generic":
            for model_id in test_node.depends_on_nodes:
                model = nodes[model_id]
                name = model.name

                if data_source == DataSource.SNOWFLAKE:
                    name = name.upper()

                path = "views" if model.config.materialized == "view" else "tables"
                generator = get_datasource_generator(
                    data_source=data_source,
                    database=model.database,
                    profile=self._context.profile,
                )
                generator.set_oddrn_paths(**{"schemas": model.schema})
                yield generator.get_oddrn_by_path(path, name)
        elif test_node.test_node_type == "singular":
            # We don't support it because it doesn't contains test_metadata
            raise NotImplementedError("Singular test nodes are not supported yet")
        else:
            raise NotImplementedError(
                f"Unknown test node type: {test_node.test_node_type}"
            )


def datetime_format(date: Optional[str]) -> Optional[datetime]:
    if date:
        return datetime.strptime(date, "%Y-%m-%dT%H:%M:%S.%fZ").astimezone(tz=pytz.utc)

    return None


def parse_status(
    result: Result, test_node: TestNode
) -> tuple[QualityRunStatus, Optional[str]]:
    status = QualityRunStatus.SUCCESS
    status_reason = None

    if result.status in {"fail", "error"}:
        status = QualityRunStatus.FAILED

    if result.status == "error":
        status_reason = result.status_reason or "Error during test execution"

    if result.status == "fail":
        print(result.status_reason)
        status_reason = StatusReason().get_reason(test_node=test_node)

    return status, status_reason



from typing import Optional


class Profile:
    def __init__(self, **config) -> None:
        del config["password"]
        self._config = config

    @classmethod
    def from_dict(cls, profile: dict, target: Optional[str] = None) -> "Profile":
        target = profile.get("target", target)
        if not target:
            raise ValueError("Target must be set")
        profile = profile.get("outputs", {}).get(target, {})
        return cls(**profile)

    @property
    def type(self) -> str:
        return self._config["type"]

    @property
    def host(self) -> str:
        return self._config["host"]

    @property
    def account(self) -> Optional[str]:
        return self._config.get("account")



import json
from pathlib import Path

import yaml


def load_yaml(file_path: Path) -> dict:
    with file_path.open() as file:
        return yaml.load(file, yaml.FullLoader)


def load_json(file_path: Path) -> dict:
    with file_path.open() as file:
        return json.load(file)



from dbt.contracts.graph.nodes import TestNode
from odd_models import MetadataExtension


def get_metadata(test_node: TestNode) -> MetadataExtension:
    metadata = {
        "name": test_node.name,
        "alias": test_node.alias,
        "build_path": test_node.build_path,
        "compiled": test_node.compiled,
        "compiled_code": test_node.compiled_code,
        "compiled_path": test_node.compiled_path,
        "created_at": test_node.created_at,
        "database": test_node.database,
        "deffered": test_node.deferred,
        "description": test_node.description,
        "extra_ctes_injected": test_node.extra_ctes_injected,
        "fqn": ", ".join(test_node.fqn),
        "language": test_node.language,
        "original_file_path": test_node.original_file_path,
        "package_name": test_node.package_name,
        "path": test_node.path,
    }

    schema_url = "https://raw.githubusercontent.com/opendatadiscovery/opendatadiscovery-specification/main/specification/extensions/dbt.json#/definitions/DataQualityTestRun"
    return MetadataExtension(schema_url=schema_url, metadata=metadata)



from pathlib import Path
from typing import Optional

from odd_dbt.errors import DbtInternalError
from odd_dbt.models import Manifest, Profile, RunResults
from odd_dbt.models.project import Project
from odd_dbt.utils import load_json, load_yaml

from .logger import logger


class DbtContext:
    run_results: RunResults
    manifest: Manifest
    profile: Profile

    def __init__(
        self,
        project_dir: Path,
        profile: Optional[str] = None,
        target: Optional[str] = None,
        profiles_dir: Optional[Path] = None,
    ):
        try:
            project = Project.parse_obj(load_yaml(project_dir / "dbt_project.yml"))

            if not (target_path := project.target_path):
                raise ValueError("Target path must be set")

            target_path = project_dir / target_path
            manifest = Manifest(target_path / "manifest.json")
            run_results = RunResults(target_path / "run_results.json")

            self.catalog = None
            if (catalog := target_path / "catalog.json").is_file():
                self.catalog = load_json(catalog)

            self.profile = self.parse_profile(
                project=project,
                profile=profile,
                target=target,
                profiles_dir=profiles_dir or run_results.profiles_dir,
            )
            self.run_results = run_results
            self.manifest = manifest
        except Exception as e:
            logger.debug(
                f"Was run with {profiles_dir=}, {profile=}, {target=}, {project_dir=}"
            )
            raise DbtInternalError(f"Failed to parse dbt context: {e}") from e

    def parse_profile(
        self,
        project: Project,
        profiles_dir: Path,
        profile: Optional[str],
        target: Optional[str],
    ) -> Profile:
        profile = profile or project.profile

        if not profile:
            raise ValueError("Profile was not found.")

        profiles = load_yaml(profiles_dir / "profiles.yml")

        if not (profile := profiles.get(profile)):
            raise ValueError("Profile was not set")

        return Profile.from_dict(profile, target)

    @property
    def results(self) -> list[dict]:
        return self.run_results.results

    @property
    def invocation_id(self) -> str:
        return self.run_results.invocation_id



from functools import cached_property
from pathlib import Path

from funcy import lmap

from odd_dbt.utils import load_json

from .result import Result


class RunResults:
    def __init__(self, file: Path) -> None:
        self._run_results = load_json(file)

    @cached_property
    def results(self):
        return lmap(Result, self._run_results.get("results", []))

    @property
    def invocation_id(self):
        return self._run_results["metadata"]["invocation_id"]

    @property
    def profiles_dir(self) -> Path:
        return Path(self._run_results["args"]["profiles_dir"])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['odd_dbt', 'odd_dbt.mapper', 'odd_dbt.models', 'odd_dbt.utils']

package_data = \
{'': ['*']}

install_requires = \
['dbt-core>=1.4.5,<2.0.0',
 'dbt-postgres==1.4.5',
 'dbt-redshift==1.4.0',
 'dbt-snowflake>=1.4.1,<2.0.0',
 'funcy>=1.17,<2.0',
 'loguru>=0.6.0,<0.7.0',
 'odd-models>=2.0.24,<3.0.0',
 'oddrn-generator>=0.1.70,<0.2.0',
 'psycopg2-binary>=2.9.6,<3.0.0',
 'sqlalchemy>=1.4.46,<2.0.0',
 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['odd_dbt_test = odd_dbt.app:app']}

setup_kwargs = {
    'name': 'odd-dbt',
    'version': '0.1.21',
    'description': 'OpenDataDiscovery Action for dbt',
    'long_description': '# OpenDataDiscovery dbt tests metadata collecting\n[![PyPI version](https://badge.fury.io/py/odd-dbt.svg)](https://badge.fury.io/py/odd-dbt)\n\nCLI tool helps automatically parse and ingest DBT test results to OpenDataDiscovery Platform.\nIt can be used as separated CLI tool or within [ODD CLI](https://github.com/opendatadiscovery/odd-cli) package which provides some useful additional features.\n\n## Installation\n```pip install odd-dbt```\n\n## Command options\n```\n╭─ Options ─────────────────────────────────────────────────────────────╮\n│    --project-dir                 PATH  [default: Path().cwd()odd-dbt] │\n│    --target                      TEXT  [default:None]                 │\n│    --profile-name                TEXT  [default:None]                 │\n│ *  --host    -h                  TEXT  [env var: ODD_PLATFORM_HOST]   │\n│ *  --token   -t                  TEXT  [env var: ODD_PLATFORM_TOKEN]  │\n│    --dbt-host                    TEXT  [default: localhost]           │\n│    --help                              Show this message and exit.    │\n╰───────────────────────────────────────────────────────────────────────╯\n```\n\n\n## Command run example\nHow to create [collector token](https://docs.opendatadiscovery.org/configuration-and-deployment/trylocally#create-collector-entity)?\n```bash\nodd_dbt_test --host http://localhost:8080 --token <COLLECTOR_TOKEN>\n```\n\n\n\n## Supported data sources\n| Source    |       |\n| --------- | ----- |\n| Snowflake | 1.4.1 |\n| Redshift  | 1.4.0 |\n| Postgres  | 1.4.5 |\n| MSSQL     |       | \n\n## Requirements\nLibrary to inject Quality Tests entities requires presence of corresponding with them datasets entities in the platform.\nFor example: if you want to inject data quality test of Snowflake table, you need to have entity of that table present in the platform.\n\n## Supported tests\nLibrary supports for basics tests provided by dbt.\n- `unique`: values in the column should be unique\n- `not_null`: values in the column should not contain null values\n- `accepted_values`: column should only contain values from list specified in the test config\n- `relationships`: each value in the select column of the model exists as a specified field in the reference table (also known as referential integrity)\n\n## ODDRN generation for datasets\n`host_settings` of ODDRN generators required for source datasets are loaded from `.dbt/profiles.yml`.\n\nProfiles inside the file looks different for each type of data source.\n\n**Snowflake** host_settings value is created from field `account`. Field value should be `<account_identifier>`\nFor example the URL for an account uses the following format: `<account_identifier>`.snowflakecomputing.com\nExample Snowflake account identifier `hj1234.eu-central-1`.\n\n**Redshift** and **Postgres** host_settings are loaded from field `host` field.\n\nExample Redshift host: `redshift-cluster-example.123456789.eu-central-1.redshift.amazonaws.com`\n',
    'author': 'Mateusz Kulas',
    'author_email': 'mkulas@provectus.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from enum import Enum
from typing import Type

from oddrn_generator import (
    Generator,
    MssqlGenerator,
    PostgresqlGenerator,
    RedshiftGenerator,
    SnowflakeGenerator,
)

from odd_dbt.models import Profile


class DataSource(Enum):
    SNOWFLAKE = "snowflake"
    REDSHIFT = "redshift"
    POSTGRES = "postgres"
    MSSQL = "mssql"


class GeneratorAdaptee:
    def __init__(
        self,
        generator_cls: Type[Generator],
    ) -> None:
        self._generator_cls = generator_cls

    def create(self, database: str, profile: Profile) -> Generator:
        host = profile.host
        return self._create(database, host)

    def _create(self, database: str, host: str) -> Generator:
        return self._generator_cls(host_settings=host, databases=database)


class SnowflakeAdaptee(GeneratorAdaptee):
    def create(self, database: str, profile: Profile) -> Generator:
        host = f"{profile.account}.snowflakecomputing.com"
        return super()._create(database, host)


DATA_SOURCE_GENERATORS: dict[DataSource, GeneratorAdaptee] = {
    DataSource.POSTGRES: GeneratorAdaptee(PostgresqlGenerator),
    DataSource.SNOWFLAKE: SnowflakeAdaptee(SnowflakeGenerator),
    DataSource.REDSHIFT: GeneratorAdaptee(RedshiftGenerator),
    DataSource.MSSQL: GeneratorAdaptee(MssqlGenerator),
}


def get_datasource_generator(
    data_source: DataSource, database: str, profile: Profile
) -> Generator:
    if generator := DATA_SOURCE_GENERATORS.get(data_source):
        return generator.create(database=database, profile=profile)
    else:
        raise ValueError(
            f"Unknown {data_source=}. Available: {DATA_SOURCE_GENERATORS.keys()}"
        )
